#---------------------------------------------------------------
#
# OCCInternetSecureScriptTemplate.pl	- code part of OCC script
#
# Copyright (c) Actinic Software Ltd 2001 All rights reserved
#
# *** Do not change this code unless you know what you are doing ***
#
# Adapted from OCCAuthorizeNetScriptTemplate.pl by Bill Birthisel
# Written by Mike Purnell
#
# This script is called by an eval() function and it will already
# have the following variables set up:
#
#	Expects:		$::sOrderNumber		- the alphanumeric order number for this order
#					$::nOrderTotal			- the total for this order (stored in based currency format e.g. 1000 = $10.00)
#					%::PriceFormatBlob   - the price format data
#					%::InvoiceContact		- customer invoice contact information
#					%::OCCShipData			- customer invoice shipping information
#					$::sCallBackURLAuth	- the URL of the authorization callback script
#					$::sCallBackURLBack	- the URL of the backup script
#					$::sCallBackURLUser	- the URL of the receipt script
#					$::sPath					- the path to the Catalog directory
#					$::sWebSiteUrl			- the Catalog web site URL
#					$::sContentUrl			- the content URL
#					$::sCartID			   - the cart id
#
#	Affects:		$::eStatus     		- the status of the transaction:
#						$::FAILURE 	- Failure
#						$::ACCEPTED - Accepted
#						$::REJECTED - Rejected
#						$::PENDING  - Pending
#					$::sErrorMessage		- error message if any
#					$::sHTML					- the HTML to display
#					$::sRedirectURL		- (optional) data for logging or
#                                     for redirect if implemented someday
#
#  $Revision: 17600 $
#
#---------------------------------------------------------------

use strict;
# 
# This block of code isolates the plug-in scripts from version specific code that they have historically used
# BEGIN
my $sCartID;
if (defined $::EC_MAJOR_VERSION)						# EC version 6 or greater
	{
	$sCartID = $::sCartID;								# Use the value setup by OrderScript.pl
	}
else															# Pre-version 6
	{
	$sCartID = $::g_sCartId;							# copy the global value
	}
# END
# This block of code isolates the plug-in scripts from version specific code that they have historically used
# 
$::eStatus = $::PENDING;								# The OCC plug-in runs in pending mode.  This script does not
																# perform the transaction.  Rather, it forwards the customer to
																# the OCC site for completion.
my (%VarTable);
$VarTable{$::VARPREFIX . 'OCC_URL'} =				# insert the OCC web site URL into the HTML template
	$sProcessScriptURL;
#
# constants
#
my $ALWAYS = 1;
my $OPTION = 0;
my $NOENCODE = 0;
my $ENCODE = 1;		# encode both bounce and redirect
my $REDIRECT = 2;		# encode redirect only
#
# Supported ISO country codes
#
my %Countries =
	(
	'canada' => 'CA',
	'united states' => 'US',
	'australia' => 'AU',
	'aruba' => 'AW',
	'belgium' => 'BE',
	'brazil' => 'BR',
	'cyprus' => 'CY',
	'germany' => 'DE',
	'spain' => 'ES',
	'france' => 'FR',
	'uk' => 'GB',
	'united kingdom' => 'GB',
	'hong kong' => 'HK',
	'japan' => 'JP',
	'south korea' => 'KR',
	'netherlands' => 'NL',
	'norway' => 'NO',
	'new zealand' => 'NZ',
	'singapore' => 'SG',
	'venezuela' => 'VE',
	);
#
# Supported ISO state/province codes
#
my %States =
	(
	'alberta' => 'AB',
	'alaska' => 'AK',
	'alabama' => 'AL',
	'arkansas' => 'AR',
	'arizona' => 'AZ',
	'british columbia' => 'BC',
	'california' => 'CA',
	'colorado' => 'CO',
	'connecticut' => 'CT',
	'washington d.c.' => 'DC',
	'delaware' => 'DE',
	'florida' => 'FL',
	'georgia' => 'GA',
	'hawaii' => 'HI',
	'iowa' => 'IA',
	'idaho' => 'ID',
	'illinois' => 'IL',
	'indiana' => 'IN',
	'kansas' => 'KS',
	'kentucky' => 'KY',
	'louisiana' => 'LA',
	'massachusetts' => 'MA',
	'manitoba' => 'MB',
	'maryland' => 'MD',
	'maine' => 'ME',
	'michigan' => 'MI',
	'minnesota' => 'MN',
	'missouri' => 'MO',
	'mississippi' => 'MS',
	'montana' => 'MT',
	'new brunswick' => 'NB',
	'north carolina' => 'NC',
	'north dakota' => 'ND',
	'nebraska' => 'NE',
	'newfoundland' => 'NF',
	'new hampshire' => 'NH',
	'new jersey' => 'NJ',
	'new mexico' => 'NM',
	'nova scotia' => 'NS',
	'northwest territories' => 'NT',
	'nunavut' => 'NU',
	'nevada' => 'NV',
	'new york' => 'NY',
	'ohio' => 'OH',
	'oklahoma' => 'OK',
	'ontario' => 'ON',
	'oregon' => 'OR',
	'pennsylvania' => 'PA',
	'prince edward island' => 'PE',
	'puerto rico' => 'PR',
	'quebec' => 'QC',
	'rhode island' => 'RI',
	'south carolina' => 'SC',
	'south dakota' => 'SD',
	'saskatchewan' => 'SK',
	'tennessee' => 'TN',
	'texas' => 'TX',
	'utah' => 'UT',
	'virginia' => 'VA',
	'vermont' => 'VT',
	'washington' => 'WA',
	'wisconsin' => 'WI',
	'west virginia' => 'WV',
	'wyoming' => 'WY',
	'yukon' => 'YT',
	);
#
# build up a string of all of the values that must be passed to OCC
#
my $sHiddenValues;
my $sRedirectData = $sProcessScriptURL;
#
# sprintf template for data form sent to the OCC
#
my $OCC_template = '<INPUT TYPE=HIDDEN NAME="%s" VALUE="%s">'."\n";
my $POST_template = '%s%s=%s';

# AddPostValues ($sPostSeparator, $sPostName, $sPostValue, $bAlways, $bEncode)
#
# adds a name/value pair to $sHiddenValues and $sRedirectData
#
sub AddPostValues
	{
	my ($sDelim, $sName, $sValue, $bAlways, $eEncode) = @_;
	if ($sValue || $bAlways)
		{
		my $sEncValue = $sValue;
		if ($eEncode)
			{
			$sEncValue = ACTINIC::EncodeText2 ($sValue, $::FALSE);
			}
		if ($eEncode == $REDIRECT)
			{
			$sHiddenValues .= sprintf ($OCC_template, $sName, $sValue);
			}
		else
			{
			$sHiddenValues .= sprintf ($OCC_template, $sName, $sEncValue);
			}
		$sRedirectData .= sprintf ($POST_template, $sDelim, $sName, $sEncValue);
		}
	}
#
# merchant ID is translated to MerchantNumber
#
if($sMerchantID == -1)
	{
	$sMerchantID = 'testing';
	}
AddPostValues ('?', 'MerchantNumber', $sMerchantID, $ALWAYS, $NOENCODE);
#
# Just English for now
#
AddPostValues ('&', 'language', 'english', $ALWAYS, $NOENCODE);
#
# Create '::' delimited Products field
#
my $sProductString = "Price::Qty::Code::Description::Flags|";
#
# amount is translated to decimal places
#
my $nNumDigits = $::PriceFormatBlob{"ICURRDIGITS"};	# read the currency format values
my ($nAmount, $nFactor, $sAmount);
if(defined $nNumDigits)
	{
	$nFactor = (10 ** $nNumDigits);
	}
else
	{
	$nFactor = 100;
	}
#
# Price and a fixed Quantity of 1
#
$sProductString .= sprintf("%d.%02d::1::", $::nOrderTotal / $nFactor,
														 $::nOrderTotal % $nFactor);
#
# order number is translated to Product Code so it appears on screen
# at InternetSecure
#
$sProductString .= $::sOrderNumber . "::";
#
# COMPANY_NAME plus "Cart" as description
#
$sProductString .= $$::g_pSetupBlob{COMPANY_NAME} . " Cart::";
#
# Currency and Testmode Flags
#
if ($bTestMode)
	{
	$sProductString .= "{TEST}";
	#
	# To force failed test transactions, substitute:
	# $sProductString .= "{TESTD}";
	}
if ($::PriceFormatBlob{SINTLSYMBOLS} eq 'USD')
	{
	$sProductString .= "{US}";
	}
#
# Now insert the Product String
#
AddPostValues ('&', 'Products', $sProductString, $ALWAYS, $NOENCODE);
#
AddPostValues ('&', 'xxxName', $::InvoiceContact{NAME}, $OPTION, $REDIRECT);
#
# ADDRESS2 is appended to ADDRESS1+'; '
#
if($::InvoiceContact{ADDRESS1})
	{
	my $sAddress = $::InvoiceContact{ADDRESS1};
	if($::InvoiceContact{ADDRESS2})
		{
		$sAddress .= '; ' . $::InvoiceContact{ADDRESS2};
		}
	AddPostValues ('&', 'xxxAddress', $sAddress, $OPTION, $REDIRECT);
	}
#
AddPostValues ('&', 'xxxCity', $::InvoiceContact{ADDRESS3}, $OPTION, $REDIRECT);
#
# Convert state name, if used, to ISO code
#
my $sProvince = '';
#
# Start by using ActinicLocations package if available
#
if($$::g_pSetupBlob{LOCATION_INTERFACE} > 0)
	{
	$sProvince = ActinicLocations::GetISOInvoiceRegionCode();
	}
#
# If we didn't get the state/province code, try the address field
#
if($sProvince eq '' ||
	$sProvince =~ /undefined/i)
	{
	# ADDRESS4 gets passed through if ISO lookup fails
	#
	$sProvince = lc($::InvoiceContact{ADDRESS4});
	if(exists $States{$sProvince})
		{
		$sProvince = $States{$sProvince};
		}
	}
AddPostValues ('&', 'xxxProvince', $sProvince, $OPTION, $REDIRECT);
#
AddPostValues ('&', 'xxxPostal', substr($::InvoiceContact{POSTALCODE}, 0, 10),
					$OPTION, $REDIRECT);
#
# Convert country name, if used, to ISO code
#
my $sCountry = '';
if($$::g_pSetupBlob{LOCATION_INTERFACE} > 1)
	{
	$sCountry = ActinicLocations::GetISOInvoiceCountryCode();
	}
if($sCountry eq '' ||
	$sCountry =~ /undefined/i)
	{
	# try country code from the shipping location if present (v3 method)
	#
	if ($::s_Ship_sOpaqueLocationData =~ /LocationCountry;(.*?);/i)
		{
		$sCountry = $1;
		}
	else
		{
		$sCountry = lc($::InvoiceContact{COUNTRY});
		#
		# lookup in country -> ISO hash
		#
		if (exists $Countries{$sCountry})
			{
			$sCountry = $Countries{$sCountry};
			}
		}
	}
AddPostValues ('&', 'xxxCountry', $sCountry, $OPTION, $REDIRECT);
#
AddPostValues ('&', 'xxxEmail', $::InvoiceContact{EMAIL}, $OPTION, $REDIRECT);
#
AddPostValues ('&', 'xxxPhone', $::InvoiceContact{PHONE}, $OPTION, $REDIRECT);
#
AddPostValues ('&', 'xxxCompany', $::InvoiceContact{COMPANY}, $OPTION, $REDIRECT);
#
# add the cart ID for validation
#
AddPostValues ('&', 'CARTID', $sCartID, $ALWAYS, $NOENCODE);

#
# URLs:
#
#		AUTH - the URL to create the authorization blob
#
my @Response;
#
# we need to strip the arguments from the URL and put them as
# hidden values for the POST to the authorization callback
#
my ($sURL, $sQuery) = split(/\?/, $::sCallBackURLAuth);# split query from url

my ($sPair, $sName, $sValue);
#
# split the rest of the URL into name value pairs
#
foreach $sPair(split(/&/, $sQuery))
	{
	if ($sPair ne '')										# ignore the trailing '&'
		{
		($sName, $sValue) = split(/=/, $sPair);
		AddPostValues ('&', $sName, ACTINIC::DecodeText($sValue, $ACTINIC::FORM_URL_ENCODED), $ALWAYS);
		}
	}
#
# order number is translated to INVOICE
#
AddPostValues ('&', 'ON', $::sOrderNumber, $ALWAYS, $NOENCODE);

#
# Always in pre-authorize mode, set the flag
#
AddPostValues ('&', 'PA', '1', $ALWAYS, $NOENCODE);

if ($bTestMode)											# only include the test mode flag if it is on
	{
	AddPostValues ('&', 'TM', '1', $ALWAYS, $NOENCODE);
	}
#
#		BACK - the URL to return to the Catalog checkout process
#            uses reserved InternetSecure variable name
#
AddPostValues ('&', 'xxxVar1', $::sCallBackURLBack, $ALWAYS, $ENCODE);

#
#		USER - the URL to the receipt script
#            uses reserved InternetSecure variable name
#
AddPostValues ('&', 'ReturnCGI', $::sCallBackURLUser, $ALWAYS, $REDIRECT);
AddPostValues ('&', 'ACT_POSTPROCESS', '1', $ALWAYS, $NOENCODE);

### ACTINIC::RecordErrors ("Debug: $sHiddenValues", $::sPath);
#
# add the OCC values to the template
#
$VarTable{$::VARPREFIX . 'OCC_VALUES'} = $sHiddenValues;

my $sLinkHTML = 'occlink.html';
if(defined $::g_pPaymentList)
	{
	$sLinkHTML = $$::g_pPaymentList{ActinicOrder::PaymentStringToEnum($::g_PaymentInfo{'METHOD'})}{BOUNCE_HTML};
	}
@Response = ACTINIC::TemplateFile($::sPath . $sLinkHTML, \%VarTable); # build the file

if ($Response[0] != $::SUCCESS)
	{
	$::eStatus = $::FAILURE;							# return a plug-in error
	$::sErrorMessage = $Response[1];
	return ($::SUCCESS);									# always return success if the script runs
	}

@Response = ACTINIC::MakeLinksAbsolute($Response[2], $::sWebSiteUrl, $::sContentUrl);
if ($Response[0] != $::SUCCESS)
	{
	$::eStatus = $::FAILURE;							# return a plug-in error
	$::sErrorMessage = $Response[1];
	return ($::SUCCESS);									# always return success if the script runs
	}

$::sHTML = $Response[2];								# grab the resulting HTML
$::sRedirectURL = $sRedirectData;					# data for log or redirect
#
# process the test mode warning
#
my ($sDelimiter) = $::DELPREFIX . 'TESTMODE';
if ($bTestMode)											# only include the test mode block if we are in test mode
	{
	$::sHTML =~ s/$sDelimiter//g;						# remove the delimiter text
	}
else															# not in test mode - remove the block
	{
	$::sHTML =~ s/$sDelimiter(.*?)$sDelimiter//gs;	# remove the test mode warning blob (/s removes the \n limitation of .)
	}

return ($::SUCCESS);

#
# End of OCCInternetSecureScriptTemplate.pl
#
